package gf;
#
#Perl implementation of arithmetical operations in a Galois Field
#Copyright (C) 2006 J. Wells
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without modification,
#are permitted provided that the following conditions are met:
#1. Redistributions of source code shall retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form shall reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#3. The name of the author may not be used to endorse or promote products
#   derived from this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#This implementation is intended to provide an easy to follow illustration.
#It is designed for transparency, not efficiency!
#
#It dies if an attempt is made to divide by zero!!!
#
#
#usage:
	#prior to use, the Galois field size has to be defined in $gf::field and
	#the log and antilog tables to use defined in $gf::log and $gf::alog
	#associated package rs provides an example of this
	#
	#$result = gf::sum($value1, $value2);	#addition of $value1 and $value2
	#$result = gf::diff($value1, $value2);	#subtraction of $value2 from $value1
	#$result = gf::prod($value1, $value2);	#product of $value1 and $value2
	#$result = gf::div($value1, $value2);	#division of $value1 by $value2

sub sum {				#galois addition is exclusive or
	my $v1 = shift;
	my $v2 = shift;
	return ($v1 ^ $v2);
}

sub diff {				#galois subtraction is exclusive or too!
					#so $v1 - $v2 is the same as $v2 - $v1
					#and -$v is the same as $v
	my $v1 = shift;
	my $v2 = shift;
	return ($v1 ^ $v2);
}

sub prod {				#galois multiply is antilog of modulus $field sum of the logs
	my $v1 = shift;
	my $v2 = shift;
	if (($v1 == 0) || ($v2 == 0)) {return 0}
	return $alog[($log[$v1] + $log[$v2]) % $field];
}

sub div {				#galois divide is antilog of modulus $field difference between logs
	my $v1 = shift;
	my $v2 = shift;
	if ($v2 == 0) {die}		#shouldn't go dividing by 0!
	elsif ($v1 == 0) {return 0}	#0 / anything is still 0
	return $alog[($log[$v1] - $log[$v2] + $field) % $field];
}

1;	#so use succeeds if the package is put in a separate .pm file
